#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Api::ProjectsController < Api::RestController

  def index
    joins = params[:includelinks] == 'true' ? [:snapshots, :project_links] : [:snapshots]
    includechildren = params[:includechildren] == 'true' ? '' : ' AND root_snapshot_id IS NULL '
    conditions = ['root_id IS NULL AND projects.scope=:scope AND enabled=:enabled AND islast=:islast' + includechildren,
      {:scope => Project::SCOPE_PROJECT, :enabled => true, :islast => true}]
    projects = Project.find(:all, :conditions => conditions, :include => joins)
    rest_render(projects)
  end

  def show
    if params[:includechildren] == 'true'
      root_project = Project.find(:first, :include => [:snapshots],
        :conditions => ['kee=:key AND enabled=:enabled AND islast=:islast', 
          {:key => params[:id], :enabled => true, :islast => true}])
      if root_project.nil?
        rest_status_ko("Project [" + params[:id] + "] doesn't exist", 404)
        return
      end
      snapshot_id = root_project.snapshots[0].id

      joins = params[:includelinks] == 'true' ? [:snapshots, :project_links] : [:snapshots]
      projects = Project.find(:all, :include => joins,
        :conditions => ["snapshots.parent_snapshot_id=:parent_snapshot_id AND islast=:islast AND projects.scope=:scope",
          {:parent_snapshot_id => snapshot_id, :enabled => true, :islast => true, :scope => Project::SCOPE_PROJECT}])
      projects.insert(0, root_project)
      rest_render(projects)
    else
      conditions =  {:kee => params[:id], :enabled => true}
      if params[:includelinks] == 'true'
        project = Project.find(:first, :conditions => conditions, :include => 'project_links')
      else
        project = Project.find(:first, :conditions => conditions)
      end

      if project.nil?
        rest_status_ko("Project [" + params[:id] + "] doesn't exist", 404)
      else
        rest_render([project])
      end
    end
  end


  protected

  def rest_to_json(projects)
    json = projects.collect{|project| project.to_hash_json(params)}.to_json
    json = params['callback'] + '(' + json + ');' if params['callback']
    json
  end

  def rest_to_xml(projects)
    xml = Builder::XmlMarkup.new(:indent => 0)
    xml.instruct!
    xml.projects do
      projects.each do |project|
        xml << project.to_xml(params)
      end
    end
  end
end
