#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
require 'fastercsv'

class Api::RulesController < Api::RestController

  def index
    language = params[:language]
    profile, mandatory, optional, inactive = nil    
    if language && !language.empty?
      plugin_keys = java_facade.getPluginsByRuleLanguage(language).collect { |plug| plug.getKey() }
      rules = Rule.search(plugin_keys, params)
      profile_name = params[:profile]
      if profile_name && !profile_name.empty?
        profile = RulesProfile.find_by_name_and_language(profile_name, language)
        if profile
          if params[:levels]
            levels = params[:levels].split(",")
            mandatory = levels.include?("mandatory")
            optional = levels.include?("optional")
            inactive = levels.include?("inactive")
          else
            mandatory = true
            optional = true
            inactive = true            
          end
          rules = rules.select do |rule|
            active_rule = profile.active_by_rule_id(rule.id)
            ((inactive and active_rule.nil?) or (active_rule and ((mandatory and active_rule.error?) or (optional and active_rule.warning?))))
          end
        else
          rest_status_ko("Profile "+ profile_name + " not found.", 404)
          return
        end
      end
      rest_render(rules, profile)
    else
      rest_status_ko("Language parameter is needed.", 400)
      return
    end
  end


  private

  def rest_render(rules, profile)
    respond_to do |format|
      format.json{ render :json => rest_to_json(rules, profile) }
      format.xml { render :xml => rest_to_xml(rules, profile) }
      format.csv {
        send_data(rest_to_csv(rules, profile),
          :type => 'text/csv; charset=utf-8; header=present',
          :disposition => 'attachment; filename=rules.csv')
      }
    end
  end

  def rest_to_json(rules, profile)
    json = rules.collect{|rule| rule.to_hash_json(profile)}.to_json
    json = params['callback'] + '(' + json + ');' if params['callback']
    json
  end

  def rest_to_xml(rules, profile)
    xml = Builder::XmlMarkup.new(:indent => 0)
    xml.instruct!
    xml.rules do
      rules.each do |rule|
        xml << rule.to_xml(profile)
      end
    end
  end

  def rest_to_csv(rules, profile)
    FasterCSV.generate do |csv|
      header = ["title", "key", "category", "plugin"]
      header << "level" if profile
      csv << header
      rules.each do |rule|
        csv << rule.to_csv(profile)
      end
    end
  end

end