#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
require 'cgi'

class RulesConfigurationController < ApplicationController

  SECTION=Navigation::SECTION_CONFIGURATION

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => ['activate_rule', 'update_param' ], :redirect_to => { :action => 'index' }

  require_role Role::ADMIN, :except => [ 'index', 'export' ]

  def index
    if params[:id]
      if params[:id].to_i<0
        redirect_to :controller => 'rules_profile'
        return
      end
      @profile = RulesProfile.find(params[:id].to_i)
    else
      @profile = RulesProfile.default_profile
    end
    init_params

    @select_plugins = [""] + java_facade.getPluginsByRuleLanguage(@profile.language).collect { |plugin| [plugin.getName(), plugin.getKey()]}
    @select_categories = [""] + RulesCategory.all.collect {|rc| [ rc.name, rc.name ] }.sort

    plugin_keys = java_facade.getPluginsByRuleLanguage(@profile.language).collect { |plug| plug.getKey() }
    @rules = Rule.search(plugin_keys, {:plugin => @plugin, :category => @category, :searchtext => @search_text}).select do |rule|
      active_rule = @profile.active_by_rule_id(rule.id)
      ((@inactive and active_rule.nil?) or (active_rule and ((@mandatory and active_rule.error?) or (@optional and active_rule.warning?))))      
    end
  end

  def export
    name = CGI::unescape(params[:name])
    language = params[:language]
    plugin_key = params[:plugin]
    if (name != 'active')
      profile = RulesProfile.find_by_name_and_language(name, language)
    else
      profile = RulesProfile.find_active_profile_by_language(language)
    end
    result = java_facade.exportConfiguration(plugin_key, profile.id)
    send_data(result, :type => 'text/xml', :disposition => 'inline')
  end

  def activate_rule
    profile = RulesProfile.find(params[:id].to_i)
    if not profile.provided?
      rule=Rule.find(params[:rule_id].to_i)
      level=params[:level].to_i
      active_rule=profile.active_by_rule_id(rule.id)
      if level<0
        active_rule.destroy if active_rule
        active_rule=nil
      else
        if active_rule.nil?
          active_rule = ActiveRule.new(:profile_id => profile.id, :rule => rule)
        end
        active_rule.failure_level=level
        active_rule.save
      end
      profile.active_rules(true) #reload the rules

      render :update do |page|
        page.replace_html("rule_#{rule.id}", :partial => 'rule', :object => rule, :locals => {:profile => profile, :active_rule => active_rule})
      end
    end
  end

  def update_param
    profile = RulesProfile.find(params[:profile_id].to_i)
    rule_param = RulesParameter.find(params[:param_id].to_i)
    active_rule = ActiveRule.find(params[:active_rule_id].to_i)
    active_param = ActiveRuleParameter.find(params[:id].to_i) if params[:id].to_i > 0
    value = params[:value]
    if not profile.provided?
      if value != ""
        active_param = ActiveRuleParameter.new(:rules_parameter => rule_param, :active_rule => active_rule ) if active_param.nil?
        active_param.value = value
        active_param.save
        active_param.valid?
        active_param.reload
      elsif not active_param.nil?
        active_param.destroy
        active_param = nil
      end
    end
    render :partial => 'rule_param', :object => nil,
      :locals => {:parameter => rule_param, :active_parameter => active_param, :profile => profile, :active_rule => active_rule }
  end

  protected

  def select_category(categories, categ_id_string)
    return categories.first if categ_id_string.nil? or categ_id_string.blank?
    categories.each do |categ|
      if categ_id_string==categ.id.to_s
        return categ
      end
    end
    nil
  end

  def init_params
    @id = params[:id]
    @plugin = params[:plugin]
    @category = params[:category]
    @mandatory = params[:mandatory]
    @optional = params[:optional]
    @inactive = params[:inactive]
    @search_text = params[:searchtext]

    # On first call, display only activated rules
    if params.size == 3
      @mandatory = "1"
      @optional = "1"
      @inactive = nil
      @plugin = ""
      @category = ""
      @search_text = ""
    end

    # If no level selected, then we activated them all
    if (@mandatory.nil? and @optional.nil? and @inactive.nil?)
      @mandatory = "1"
      @optional = "1"
      @inactive = "1"
    end
  end
end