#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
module ProjectsHelper
  include ActionView::Helpers::UrlHelper

  def get_column_content(column, snapshot, measures_by_snapshot)
    project = snapshot.project
    content = ''
    nowrap = true
    measure_param = '0'
    
    if column.project_column?
      url = request.relative_url_root + "/project/index/#{project.key}"
      content = link_to project.fullname, url, :id => "dashboard_link_#{project.id}"
      measure_param=h project.fullname
      nowrap = false

    elsif column.language_column?
      content = snapshot.qualifier_label || ''

    elsif column.version_column?
      content = snapshot.version || ''

    elsif column.links_column?
      links = links_by_project_id[project.id]
      if links
        links.each do |link|
          content << link_to(image_tag(link.icon, :alt => link.full_name), link.href, :class => 'nolink', :popup => true) if (link.display? and !link.custom?)
        end
      end
    elsif column.build_time_column?
      measure_param = snapshot.created_at_in_secs
      content = human_short_date( snapshot.created_at )

    elsif column.iso_rules_compliance_column?
      measure = item_by_metric_name_and_categ_id(measures_by_snapshot[snapshot], Metric::RULES_MANDATORY_INDEX, column.id)
      measure_param = measure.value if measure
      content = measure.formatted_value if measure
      content << tendency_icon(measure, false, '&nbsp;&nbsp;&nbsp;&nbsp;')

    elsif column.metric_column?
      measure = item_by_metric_id(measures_by_snapshot[snapshot], column.id )
      measure_param = measure.value if measure
      content=html_measure(measure, nil, true, false, nil, true)

    end

    "<td #{"nowrap='nowrap'" if nowrap} #{"x='#{measure_param}'" if measure_param} class='#{column.get_column_css}'>" + content + "</td>"
  end

  def monitored_to_sentence(monitored_list)
    monitored_list.collect {|s|
      name = s.getName()
      time = time_ago_in_words(Time.at(s.getTime()/1000))
      "<b>#{name}</b> (for #{time})"
    }.to_sentence
  end

  def search_measure(measures, metric_key)
    item_by_metric_id(measures, metric_key )
  end

  private

  def item_by_metric_id(items, metric_name)
    return nil if items.nil?
    items.each do |item|
      metric = Metric.by_name(metric_name)
      return item if (item.metric.id==metric.id and item.rules_category_id.nil?)
    end
    nil
  end

  def item_by_metric_name_and_categ_id(items, metric_name, rules_category_name)
    return nil if items.nil?
    items.each do |item|
      rules_category = RulesCategory.by_name(rules_category_name)
      return item if (item.metric.name==metric_name and
        item.rules_category_id == rules_category.id and
        item.rule_id.nil?)
    end
    nil
  end

  def links_by_project_id
    @links_by_project_id ||= {}
    if @links_by_project_id.empty?
      ProjectLink.find(:all, :order => 'link_type').each do |link|
        @links_by_project_id[link.project_id] ||= []
        @links_by_project_id[link.project_id]<<link
      end
    end
    @links_by_project_id
  end

end
