#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Rule < ActiveRecord::Base

  validates_presence_of :name, :rules_category_id, :plugin_rule_key, :plugin_config_key, :plugin_name

  has_many :rules_parameter, :dependent => :destroy
  belongs_to :rules_category
  has_many :rule_failures

  has_many :active_rules

  def parameters
    rules_parameter
  end
  
  def parameter(name)
    result=nil
    parameters.each do |param|
      result = param if param.name==name
    end
    result
  end
  
  def rules_category
    category
  end

  def category
    @rules_category ||=
      begin
      RulesCategory.by_id(rules_category_id)
    end
  end

  def category=(c)
    @rules_category = c
  end

  def to_hash_json(profile)
    json = {'title' => name, 'key' => plugin_rule_key, 'category' => rules_category.name, 'plugin' => plugin_name}
    json['description'] = description
    active_rule = nil
    if profile
      active_rule = profile.active_by_rule_id(id)
      if active_rule
        json['level'] = "mandatory" if active_rule.error?
        json['level'] = "optional" if active_rule.warning?
      else
        json['level'] = "inactive" if active_rule.nil?
      end
    end
    json['params'] = parameters.collect{|parameter| parameter.to_hash_json(active_rule)} if not parameters.empty?
    json
  end

  def to_xml(profile)
    xml = Builder::XmlMarkup.new
    xml.rule do
      xml.title(name)
      xml.key(plugin_rule_key)
      xml.category(rules_category.name)
      xml.plugin(plugin_name)
      xml.description {xml.cdata!(description)}
      active_rule = nil
      if profile
        active_rule = profile.active_by_rule_id(id)
        if active_rule
          xml.level("mandatory") if active_rule.error?
          xml.level("optional") if active_rule.warning?
        else
          xml.level("inactive")
        end
      end
      parameters.each do |parameter|
        xml << parameter.to_xml(active_rule)
      end
    end
  end

  def to_csv(profile)
    csv = [name, plugin_rule_key, rules_category.name, plugin_name]
    if profile
      active_rule = profile.active_by_rule_id(id)
      if active_rule
        csv << "mandatory" if active_rule.error?
        csv << "optional" if active_rule.warning?
      else
        csv << "inactive" if active_rule.nil?
      end      
    end
    csv
  end


  # Options can only contains :plugin, :category, :search_text
  def self.search(plugin_keys, options={})
    conditions_list = []
    plugin = nil
    category_id = nil
    searchtext = options[:searchtext].nil? ? "" : options[:searchtext].to_s.strip
    search = nil
    
    if options[:plugin] and not options[:plugin].empty?
      conditions_list << "plugin_name = :plugin_name"
      plugin = options[:plugin].clone
    else
      conditions_list << "plugin_name IN (:plugin_name)"
      plugin = plugin_keys
    end
    if options[:category] and not options[:category].empty?
      category_id = RulesCategory.by_name(options[:category])
      conditions_list << "rules_category_id = :rules_category_id"
    end
    if options[:searchtext] and not options[:searchtext].empty?
      search = "%" << searchtext.clone.upcase << "%"
      conditions_list << "(UPPER(rules.name) LIKE :searchtext OR plugin_rule_key = :key)"
    end

    Rule.find :all, :order => 'rules.name', :include => ["rules_parameter"],
      :conditions => [conditions_list.join(" AND "),
        {:plugin_name => plugin, :rules_category_id => category_id, :searchtext => search, :key => searchtext}
      ]
  end

end
