ActionController::Routing::Routes.draw do |map|

  map.resources :events
  map.resources :alerts
  map.resources :profiles, :has_many => :alerts

  # For ajax search
  map.connect 'api/projects/search', :controller => 'api/projects', :action => 'search'

  map.connect 'api/projects/:project_id/sources', :controller => 'api/sources', :action => 'index', :requirements => { :project_id => /.*/}
  map.connect 'api/projects/:project_id/violations', :controller => 'api/violations', :action => 'index', :requirements => { :project_id => /.*/}

  map.connect 'api/projects/:project_id/measures', :controller => 'api/measures', :action => 'index', :requirements => { :project_id => /.*/}, :conditions => { :method => :get }
  map.connect 'api/projects/:project_id/measures', :controller => 'api/measures', :action => 'create', :requirements => { :project_id => /.*/}, :conditions => { :method => :post }
  map.connect 'api/projects/measures', :controller => 'api/measures', :action => 'index'

  map.connect 'api/projects', :controller => 'api/projects', :action => 'index'
  map.connect 'api/projects/:id', :controller => 'api/projects', :action => 'show', :requirements => { :id => /.*/}

  map.connect 'api/metrics', :controller => 'api/metrics', :action => 'index', :conditions => { :method => :get }
  map.connect 'api/metrics/:id', :controller => 'api/metrics', :action => 'show', :conditions => { :method => :get }
  map.connect 'api/metrics/:id', :controller => 'api/metrics', :action => 'create', :conditions => { :method => :post }
  map.connect 'api/metrics/:id', :controller => 'api/metrics', :action => 'update', :conditions => { :method => :put }
  map.connect 'api/metrics/:id', :controller => 'api/metrics', :action => 'destroy', :conditions => { :method => :delete }

  map.resources 'rules', :path_prefix => 'api', :controller => 'api/rules'
  map.resources 'properties', :path_prefix => 'api', :controller => 'api/properties', :requirements => { :id => /.*/}

  map.connect 'api/server/:action', :controller => 'api/server'

  # home page
  map.home '', :controller => 'projects', :action => 'index'
  map.connect '', :controller => 'projects', :action => 'index'

  # views
  map.connect 'views/projects/:view', :controller => 'views/projects', :action => 'index', :requirements => { :view => /.*/}
  map.connect 'views/project/:id/:view', :controller => 'views/project', :action => 'index', :requirements => { :id => /.*/, :view => /.*/}

  # to refactor
  map.connect 'charts/:action/:project_id/:metric_id', :controller => 'charts'
  map.connect 'rules_configuration/:action/:language/:name/:plugin.:format', :controller => 'rules_configuration'

  # Install the default route as the lowest priority.
  map.connect ':controller/:action/:id', :requirements => { :id => /.*/}

end